

% A simple list implementation.
% in a real Prolog system, you would use the builtin list type [].

% The main difference to the version in class is that this code
% does the "destructuring" already in the clause heads.

% Given an element Elem and an existing list Tail,
% create a new list by linking the Element with the Tail.
list_add(Elem, Tail, NewList) :-
	NewList = list_link(Elem, Tail).

% Create a singleton list by appending the
% element to an empty list.
list_new(Elem, NewList) :-
	list_add(Elem, list_end, NewList).

% Compute/check the length of a list.
% Base case: list is simple end.
list_length(list_end, Length) :-
	Length = 0.
% Recursion: haven't seen the end yet.
list_length(list_link(_, Tail), Length) :-
	list_length(Tail, TailLength),
	Length is TailLength + 1.

% Test list membership: succeeds if Elem is part of the list.
% Try removing the cut operator (!) and see what changes.
list_contains(Elem, list_link(Head, _)) :-
	Elem = Head, !.
list_contains(Elem, list_link(_, Tail)) :-
	list_contains(Elem, Tail).

% Extract an element from a list.
% Base case: extract the head.
nth(0, list_link(Head, _), Result) :-
	Result = Head.
% Recursion: not this one, extract from the Tail.
nth(N, list_link(_, Tail), Result) :-
	Next is N - 1,
	nth(Next, Tail, Result).

